<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Implementation of Contenido_UrlBuilder to build front_content.php URL
 *
 * Requirements:
 * @con_php_req 5.0
 *
 * @package     Contenido
 * @subpackage  ModRewrite
 * @version     0.3
 * @author      Murat Purc
 * @copyright    Murat Purc 2008
 * @license     http://www.contenido.org/license/LIZENZ.txt
 * @link        http://www.4fb.de
 * @link        http://www.contenido.org
 */


if(!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


cInclude('classes', 'UrlBuilder/Contenido_UrlBuilder.class.php');
cInclude('classes', 'UrlBuilder/Contenido_UrlBuilderFactory.class.php');


/**
 * Class to build frontend urls for advandced mod rewrite plugin.
 * Extends abstract Contenido_UrlBuilder class and implements singleton pattern.
 *
 * Usage:
 * <code>
 * cInclude('classes', 'UrlBuilder/Contenido_UrlBuilder_MR.class.php');
 * $url = 'front_content.php?idart=123';
 * $mrUrlBuilder = Contenido_UrlBuilder_MR::getInstance();
 * $mrUrlBuilder->buildUrl(array($url));
 * $newUrl = $mrUrlBuilder->getUrl();
 * </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */
class Contenido_UrlBuilder_MR extends Contenido_UrlBuilder {

    /**
     * Self instance
     *
     * @var  Contenido_UrlBuilder_MR
     */
    static private $_instance;

    /**
     * Debugger instance
     *
     * @var  Contenido_mpDebug
     */
    private $_oDebug;

    /**
     * Ampersant used for composing several parameter value pairs
     *
     * @var  string
     */
    private $_sAmp = '&amp;';

    /**
     * Is XHTML output?
     *
     * @var  bool
     */
    private $_bIsXHTML = false;

    /**
     * Is mod rewrite enabled?
     *
     * @var  bool
     */
    private $_bMREnabled = false;

    /**
     * Mod Rewrite configuration
     *
     * @var  array
     */
    private $_aMrCfg = null;


    /**
     * Constructor, tries to set some member variables.
     */
    private function __construct() {
        $this->sHttpBasePath = '';
        if (ModRewrite::isEnabled()) {
            $this->_oDebug     = Contenido_mpDebug::getInstance();
            $this->_aMrCfg     = ModRewrite::getConfig();
            $this->_bMREnabled = true;
            $this->_bIsXHTML   = (getEffectiveSetting('generator', 'xhtml', 'false') == 'false') ? false : true;
            $this->_sAmp       = ($this->_bIsXHTML) ? '&' : '&amp;';
        }
    }


    /**
     * Returns a instance of Contenido_UrlBuilder_MR
     *
     * @return  Contenido_UrlBuilder_MR
     */
    public static function getInstance() {
        if (self::$_instance == null) {
            self::$_instance = new Contenido_UrlBuilder_MR();
        }
        return self::$_instance;
    }


    /**
     * Builds a URL based on defined mod rewrite settings.
     *
     * @param   array    $params  Parameter array, provides only following parameters:
     * <code>
     * $params[0] = 'front_content.php?idart=123...'
     * </code>
     * @param   boolean  $bUseAbsolutePath  Flag to use absolute path (not used at the moment)
     * @return  string   New build url
     */
    public function buildUrl(array $params, $bUseAbsolutePath=false) {
        $url = (isset($params[0])) ? $params[0] : '';

        if ($this->_bMREnabled) {

            $url  = ModRewrite::urlPreClean($url);
            $aUrl = ModRewrite::getClientFullUrlParts($url);
            if (preg_match("/^front_content\.php(.*|.+?)/i", $aUrl['url'], $aHits) == 1) {
                $url = $aUrl['htmlpath'] . self::_buildUrl($aHits[1]);
            } else {
                $this->_oDebug->addDebug($url, 'Contenido_UrlBuilder_MR::buildUrl() no match');
            }
        }

#        $urlDebug['out'] = $url;
#        $this->_oDebug->addDebug($urlDebug, 'Contenido_UrlBuilder_MR::buildUrl() in -> out');

        $this->sUrl = $url;
    }


    /**
     * Builds the SEO-URL by analyzing passed arguments (parameter value pairs)
     *
     * @param   string  $args  Arguments
     * @return  string  New build pretty url
     */
    private function _buildUrl($args='') {
        // check arguments, try to set anchor, if exists and do some cleanup
        $sAnchor = '';
        $aTmp = explode('#', $args);
        if (count($aTmp) > 1) {
            $args = $aTmp[0];
            $sAnchor = '#' . urlencode(urldecode($aTmp[1]));
        }
        $args = str_replace('?', '', $args);

        // extract arguments into arguments array
        $aArgs = array();
        parse_str($args, $aArgs);

        // set list of parameter which are to ignore while setting additional parameter
        $aIgnoredParams = array(
            'idcat', 'idart', 'lang', 'client', 'idcatart', 'idartlang'
        );
        if ($this->_aMrCfg['use_language'] == 1) {
            $aIgnoredParams[] = 'changelang';
        }
        if ($this->_aMrCfg['use_client'] == 1) {
            $aIgnoredParams[] = 'changeclient';
        }

        // collect additional non contenido related parameters
        $sParams = '';
        foreach ($aArgs as $p => $v) {
            if (!in_array($p, $aIgnoredParams)) {
                $sParams .= urlencode(urldecode($p)) . '=' . urlencode(urldecode($v)) . $this->_sAmp;
            }
        }
        if (strlen($sParams) > 0) {
            $sParams = '?' . substr($sParams, 0, - strlen($this->_sAmp));
        }

        // some presettings of variables
        $aParts = array();
        $iChangeClient = (isset($aArgs['changeclient'])) ? (int) $aArgs['changeclient'] : 0;
        $iChangeLang   = (isset($aArgs['changelang'])) ? (int) $aArgs['changelang'] : 0;
        $iIdCat        = (isset($aArgs['idcat'])) ? (int) $aArgs['idcat'] : 0;
        $iIdCatLang    = (isset($aArgs['idcatlang'])) ? (int) $aArgs['idcatlang'] : 0;
        $iIdCatArt     = (isset($aArgs['idcatart'])) ? (int) $aArgs['idcatart'] : 0;
        $iIdArt        = (isset($aArgs['idart'])) ? (int) $aArgs['idart'] : 0;
        $iIdArtLang    = (isset($aArgs['idartlang'])) ? (int) $aArgs['idartlang'] : 0;

        // set client if desired
        if ($this->_aMrCfg['use_client'] == 1) {
            $idclient = ($iChangeClient > 0) ? $iChangeClient : $GLOBALS['client'];
            if ($this->_aMrCfg['use_client_name'] == 1) {
                $aParts[] = urlencode(ModRewrite::getClientName($idclient));
            } else {
                $aParts[] = $idclient;
            }
        }

        // set language if desired
        if ($this->_aMrCfg['use_language'] == 1) {
            $idlang = ($iChangeLang > 0) ? $iChangeLang : $GLOBALS['lang'];
            if ($this->_aMrCfg['use_language_name'] == 1) {
                $aParts[] = urlencode(ModRewrite::getLanguageName($idlang));
            } else {
                $aParts[] = $idlang;
            }
        }

        // get pretty url parts
        $oMRUrlStack = ModRewriteUrlStack::getInstance();
        $aPretty = $oMRUrlStack->getPrettyUrlParts('front_content.php?' . $args);

        $sPath    = (isset($aPretty['urlpath'])) ? $aPretty['urlpath'] : '';
        $sArticle = (isset($aPretty['urlname'])) ? $aPretty['urlname'] : '';

        if (strlen($sPath) > 0) {
            $aParts[] = $sPath;
        }
        $sPath = implode('/', $aParts);

        // check start directory settings
        if ($this->_aMrCfg['startfromroot'] == 0) {
            // splitt string in array
##++##            $aCategories = explode($this->_aMrCfg['category_seperator'], $sPath);
            $aCategories = explode('/', $sPath);

            // remove first category
            array_shift($aCategories);

            // implode array with categories to new string
##++##            $sPath = implode($this->_aMrCfg['category_seperator'], $aCategories);
            $sPath = implode('/', $aCategories);
        }

        // category id was passed but not article id
        if (($iIdCat > 0 || $iIdCatLang > 0) && $iIdCatArt == 0 && $iIdArt == 0 && $iIdArtLang == 0) {
            $sArticle = '';
            if ($this->_aMrCfg['add_startart_name_to_url']) {
                if ($this->_aMrCfg['default_startart_name'] !== '') {
                    // use default start article name
                    $sArticle = $this->_aMrCfg['default_startart_name'];
                }
            } else {
                // url is to create without article name
                $sArticle = '';
            }
        }

        if ($sArticle !== '') {
            $sFileExt = $this->_aMrCfg['file_extension'];
        } else {
            $sFileExt = '';
        }

        // add closing path seperator
        $sPath .= '/';

        $sPathAndArticle = $sPath . $sArticle . $sFileExt;
        if ($this->_aMrCfg['use_lowercase_uri'] == 1) {
            // use lowercase url
            $sPathAndArticle = strtolower($sPathAndArticle);
        }

        $sFullUrl = $this->_aMrCfg['rootdir'] . $sPathAndArticle . $sParams . $sAnchor;
#echo "<pre>Contenido_UrlBuilder_MR->_buildUrl: \$sFullUrl $sFullUrl</pre>";

        // remove double or more join parameter
        $sFullUrl = mr_removeMultipleChars($this->_aMrCfg['category_seperator'], $sFullUrl);
        if (substr($sFullUrl, -2) == '?=') {
            $sFullUrl = substr_replace($sFullUrl, '', -2);
        }
#echo "<pre>Contenido_UrlBuilder_MR->_buildUrl: \$sFullUrl $sFullUrl</pre>";
        // now convert url to an AMR url
        $oMrUrlUtil = ModRewriteUrlUtil::getInstance();
        $sFullUrl = $oMrUrlUtil->toModRewriteUrl($sFullUrl);

        return $sFullUrl;
    }

}
